require 'test_helper'

class CorrespondingFileTest < Minitest::Test
  include Splot

  def test_looks_in_test_and_spec_when_file_is_in_lib
    mock_files_existing "test/my_class_test.rb" do
      assert_equal "test/my_class_test.rb",
        CorrespondingFile.fetch("lib/my_class.rb")
    end
    mock_files_existing "spec/my_class_spec.rb" do
      assert_equal "spec/my_class_spec.rb",
        CorrespondingFile.fetch("lib/my_class.rb")
    end
  end

  def test_lib_directory_priority
    mock_files_existing "test/my_class_test.rb", "test/lib/my_class_test.rb" do
      assert_equal "test/lib/my_class_test.rb",
        CorrespondingFile.fetch("lib/my_class.rb")
    end
  end

  def test_gem_lib_dir
    mock_files_existing "test/my_class_test.rb" do
      assert_equal "test/my_class_test.rb",
        CorrespondingFile.fetch("lib/gem_name/my_class.rb")
    end
  end

  def test_namespacing
    mock_files_existing "test/lib/my_namespace/my_class_test.rb" do
      assert_equal "test/lib/my_namespace/my_class_test.rb",
        CorrespondingFile.fetch("lib/my_namespace/my_class.rb")
    end
    mock_files_existing "test/lib/commands/open_bank_account_test.rb" do
      assert_equal "test/lib/commands/open_bank_account_test.rb",
        CorrespondingFile.fetch("lib/finance_fu/commands/open_bank_account.rb")
    end
  end

  def test_rails_models
    mock_files_existing "test/unit/my_model_test.rb" do
      assert_equal "test/unit/my_model_test.rb",
        CorrespondingFile.fetch("app/models/my_model.rb")
    end
    mock_files_existing "test/models/my_model_test.rb" do
      assert_equal "test/models/my_model_test.rb",
        CorrespondingFile.fetch("app/models/my_model.rb")
    end
  end

  def test_rails_controllers
    mock_files_existing "test/functional/my_controller_test.rb" do
      assert_equal "test/functional/my_controller_test.rb",
        CorrespondingFile.fetch("app/controllers/my_controller.rb")
    end
    mock_files_existing "test/controllers/my_controller_test.rb" do
      assert_equal "test/controllers/my_controller_test.rb",
        CorrespondingFile.fetch("app/controllers/my_controller.rb")
    end
  end

  def test_rails_custom_app_load_path
    mock_files_existing "test/services/my_service_test.rb" do
      assert_equal "test/services/my_service_test.rb",
        CorrespondingFile.fetch("app/services/my_service.rb")
    end
  end
end
