require "test_helper"

class SplotTest < Minitest::Test
  def test_binary
    assert_equal 0, run_binary("-r ruby --include-path test/fixtures -f test/fixtures/test_unit_example_test.rb --line 5")
    refute_equal 0, run_binary("-r ruby --include-path test/fixtures -f test/fixtures/test_unit_example_test.rb --line 8")
  end

  def test_test_unit
    assert_equal "rake test TEST=test/fixtures/test_unit_example_test.rb",
      Splot.command(file: test_unit_rb)

    assert_equal "rake test TEST=test/fixtures/test_unit_example_test.rb TESTOPTS=\"--name=test_always_passes\"",
      Splot.command(file: test_unit_rb, line: 5)
    assert_equal "rake test TEST=test/fixtures/test_unit_example_test.rb TESTOPTS=\"--name=test_always_fails\"",
      Splot.command(file: test_unit_rb, line: 8)
  end

  def test_test_unit_with_ruby_runner
    assert_equal "ruby test/fixtures/test_unit_example_test.rb",
      Splot.command(runner: "ruby", file: test_unit_rb)
  end

  def test_test_unit_with_invalid_line_numbers
    assert_equal "rake test TEST=test/fixtures/test_unit_example_test.rb",
      Splot.command(file: test_unit_rb, line: 1)
  end

  def test_activesupport_test_unit
    assert_equal "rake test TEST=test/fixtures/activesupport_example_test.rb",
      Splot.command(file: activesupport_test_rb)

    assert_equal "rake test TEST=test/fixtures/activesupport_example_test.rb TESTOPTS=\"--name=test_will_always_pass\"",
      Splot.command(file: activesupport_test_rb, line: 5)
    assert_equal "rake test TEST=test/fixtures/activesupport_example_test.rb TESTOPTS=\"--name=test_will_always_fail\"",
      Splot.command(file: activesupport_test_rb, line: 8)
  end

  def test_cucumber
    assert_equal "cucumber path/to/cucumber_example.feature",
      Splot.command(file: cucumber_feature)

    assert_equal "cucumber path/to/cucumber_example.feature -l 2",
      Splot.command(file: cucumber_feature, line: 2)
  end

  def test_rspec
    assert_equal "rspec path/to/rspec_example_spec.rb",
      Splot.command(file: rspec_spec_rb)

    assert_equal "rspec path/to/rspec_example_spec.rb -l 3",
      Splot.command(file: rspec_spec_rb, line: 3)
  end

  def test_unknown
    assert_raises Splot::RunnerNotFoundError do
      Splot.command file: "path/to/any_old_ruby_file.rb"
    end
  end

  def test_preloader
    assert_equal "spring rspec my_spec.rb",
      Splot.command(file: "my_spec.rb", preloader: "spring")
  end

  def test_adding_include_paths
    assert_equal "rake test -Imy_lib_dir TEST=test/fixtures/test_unit_example_test.rb",
      Splot.command(file: test_unit_rb, include_path: "my_lib_dir")
    assert_equal "rspec -Imy_lib_dir path/to/rspec_example_spec.rb",
      Splot.command(file: rspec_spec_rb, include_path: "my_lib_dir")
    assert_raises ArgumentError do
      assert_equal "cucumber -Imy_lib_dir TEST=path/to/cucumber_example.feature",
        Splot.command(file: cucumber_feature, include_path: "my_lib_dir")
    end
  end

  def test_lib_files_automatically_translate_to_corresponding_test
    assert_equal "rake test TEST=test/splot_test.rb",
      Splot.command(file: "lib/splot.rb")
    # Forget the line number when we change files
    assert_equal "rake test TEST=test/splot_test.rb",
      Splot.command(file: "lib/splot.rb", line: 5)
  end

  def test_config_file_can_set_params
    assert_equal "spring rspec path/to/my_spec.rb",
      Splot.command(file: "path/to/my_spec.rb", config_file: "test/fixtures/config_example_rc")
  end

  def test_auto_loads_dot_splotrc
    splotrc = File.expand_path('../../.splotrc', __FILE__)
    File.write splotrc, "include_path: test/fixtures"
    assert_equal 0, run_binary("-r ruby -f test/fixtures/test_unit_example_test.rb --line 5")
  ensure
    File.unlink splotrc if File.exist? splotrc
  end

  private

  def run_binary(command)
    Process::fork do
      command.insert 0, "RUBYLIB=#{File.expand_path('../../lib', __FILE__)}:$RUBYLIB bin/splot "
      $stdout.reopen File.open("/dev/null", "w+")
      $stderr.reopen File.open("/dev/null", "w+")
      exec command
    end
    Process::wait
    $?.exitstatus
  end

  def activesupport_test_rb
    "test/fixtures/activesupport_example_test.rb"
  end

  def cucumber_feature
    "path/to/cucumber_example.feature"
  end

  def rspec_spec_rb
    "path/to/rspec_example_spec.rb"
  end

  def test_unit_rb
    "test/fixtures/test_unit_example_test.rb"
  end
end
